@@ -1,21 +1,27 @@
 import base64
 import enum
 import hashlib
+import hmac
+import ipaddress
 import json
+import os
 import random
 import re
+import time
 import uuid
 from _decimal import Decimal
 from datetime import datetime, timedelta, timezone
 from typing import List, Union, Any, Dict, Optional, Literal
 
 import inflection
+from cryptography.fernet import Fernet
 from fastapi import APIRouter, Path
 from fastapi.encoders import jsonable_encoder
 from fastapi.responses import Response
 from jinja2 import Template
 from jose import jwt
 from passlib.context import CryptContext
+from starlette.exceptions import HTTPException
 from starlette.requests import Request
 
 # Constants
@@ -28,6 +34,7 @@ class Utils:
     default_max_allowed_miles = 200 * 365
     pwd_context = CryptContext(schemes=["sha256_crypt"])
 
+    # Datetime
     @staticmethod
     def datetime_now() -> datetime:
         return datetime.now(timezone.utc)
@@ -346,6 +353,76 @@ class Utils:
                 f"File {file_path.name} is too large ({file_size // (1024 * 1024)}MB). Max allowed size is {MAX_ATTACHMENT_SIZE // (1024 * 1024)}MB"
             )
 
+client_secret_encryption_key = os.getenv('CLIENT_SECRET_ENCRYPTION_KEY')
+fernet = Fernet(client_secret_encryption_key)
+
+class ClientUtils:
+
+    @staticmethod
+    def get_client_ip(request: Request) -> str:
+        x_forwarded_for = request.headers.get("x-forwarded-for")
+        if x_forwarded_for:
+            # X-Forwarded-For may contain a list of IPs
+            ip = x_forwarded_for.split(",")[0].strip()
+        else:
+            ip = request.client.host
+        return ip
+
+    @staticmethod
+    def is_ip_allowed(ip: str, allowed_ips: List[str]) -> bool:
+        try:
+            client_ip = ipaddress.ip_address(ip)
+            return any(client_ip == ipaddress.ip_address(allowed) for allowed in allowed_ips)
+        except ValueError:
+            return False
+
+    @staticmethod
+    def extract_domain_from_referer_or_origin(header: Optional[str]) -> Optional[str]:
+        try:
+            from urllib.parse import urlparse
+            if header:
+                parsed = urlparse(header)
+                return parsed.hostname
+        except Exception:
+            pass
+        return None
+
+    @staticmethod
+    def encrypt_api_secret(secret: str) -> str:
+        return fernet.encrypt(secret.encode()).decode()
+
+    @staticmethod
+    def decrypt_api_secret(encrypted_secret: str) -> str:
+        return fernet.decrypt(encrypted_secret.encode()).decode()
+
+    @staticmethod
+    def compute_signature(secret, method, path, timestamp, body):
+        msg = method + path + str(timestamp) + body
+        return hmac.new(secret.encode(), msg.encode(), hashlib.sha256).hexdigest()
+
+    @staticmethod
+    async def verify_signature(request: Request, client_secret: str):
+        api_key = request.headers.get("x-api-key")
+        signature = request.headers.get("x-signature")
+        timestamp = request.headers.get("x-timestamp")
+
+        if not all([api_key, signature, timestamp]):
+            raise HTTPException(status_code=401, detail="Missing headers")
+
+        if abs(int(time.time()) - int(timestamp)) > 300:
+            raise HTTPException(status_code=401, detail="Request expired")
+
+        if not client_secret:
+            raise HTTPException(status_code=403, detail="Invalid API key")
+
+        body = await request.body()
+        path = str(request.url.path)
+        method = request.method.upper()
+
+        expected_signature = ClientUtils.compute_signature(client_secret, method, path, timestamp, body.decode())
+
+        if not hmac.compare_digest(expected_signature, signature):
+            raise HTTPException(status_code=403, detail="Invalid signature")
 
 class RouterUtils:
 
